{-|
Module      : Gargantext.Prelude.Config
Description : Textmining Collaborative Platform
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}

{-# LANGUAGE TemplateHaskell     #-}

module Gargantext.Prelude.Config where

import Control.Lens (makeLenses)
import Data.Either.Extra (Either(Left, Right))
import Data.Ini (readIniFile, lookupValue, Ini)
import Data.Text as T
import GHC.Generics (Generic)
import Prelude (read)
import System.IO (FilePath)

import Gargantext.Prelude
import Gargantext.Prelude.Mail.Types (MailConfig(..))


-- | strip a given character from end of string
stripRight :: Char -> T.Text -> T.Text
stripRight c s = if T.last s == c then stripRight c (T.take (T.length s - 1) s) else s

data GargConfig = GargConfig { _gc_url               :: !T.Text
                             , _gc_url_backend_api   :: !T.Text

                             , _gc_masteruser        :: !T.Text
                             , _gc_secretkey         :: !T.Text

                             , _gc_datafilepath      :: !FilePath
                             , _gc_repofilepath      :: !FilePath

                             , _gc_frame_write_url   :: !T.Text
                             , _gc_frame_calc_url    :: !T.Text
                             , _gc_frame_visio_url   :: !T.Text

                             , _gc_frame_searx_url   :: !T.Text
                             , _gc_frame_istex_url   :: !T.Text

                             , _gc_max_docs_parsers  :: !Integer
                             , _gc_max_docs_scrapers :: !Integer
                             }
  deriving (Generic, Show)

makeLenses ''GargConfig

readIniFile' :: FilePath -> IO Ini
readIniFile' fp = do
  ini <- readIniFile fp
  case ini of
    Left e     -> panic $ T.pack $ "ini file not found " <> show e
    Right ini' -> pure ini'

val :: Ini -> Text -> Text -> Text
val ini section key = do
  case (lookupValue section key ini) of
    Left e   -> panic $ "ERROR: add " <> key <> " in section \"" <> section <> "\" to your gargantext.ini"
    Right p' -> p'

readConfig :: FilePath -> IO GargConfig
readConfig fp = do
  ini <- readIniFile' fp

  let val' = val ini "gargantext"

  pure $ GargConfig
    { _gc_url               = stripRight '/' $ val' "URL"
    , _gc_url_backend_api   = stripRight '/' $ val' "URL_BACKEND_API"
    , _gc_masteruser        = val' "MASTER_USER"
    , _gc_secretkey         = val' "SECRET_KEY"
    , _gc_datafilepath      = cs $ val' "DATA_FILEPATH"
    , _gc_repofilepath      = cs $ val' "REPO_FILEPATH"
    , _gc_frame_write_url   = stripRight '/' $ val' "FRAME_WRITE_URL"
    , _gc_frame_calc_url    = stripRight '/' $ val' "FRAME_CALC_URL"
    , _gc_frame_visio_url   = stripRight '/' $ val' "FRAME_VISIO_URL"
    , _gc_frame_searx_url   = stripRight '/' $ val' "FRAME_SEARX_URL"
    , _gc_frame_istex_url   = stripRight '/' $ val' "FRAME_ISTEX_URL"
    , _gc_max_docs_parsers  = read $ cs $ val' "MAX_DOCS_PARSERS"
    , _gc_max_docs_scrapers = read $ cs $ val' "MAX_DOCS_SCRAPERS"
    }
